@@ -1,6 +1,6 @@
-ymatrix.mnp <- function(data, base=NULL, extra=FALSE, verbose=verbose) { 
+ymatrix.mnp <- function(data, base=NULL, extra=FALSE) {
   ## checking and formatting Y
-  Y <- model.response(data)
+  Y <- stats::model.response(data)
   if (is.matrix(Y)) { # Multinomial ordered Probit model
     for (i in 1:nrow(Y))
       Y[i,] <- match(Y[i,], sort(unique(Y[i,]))) - 1
@@ -8,14 +8,14 @@ ymatrix.mnp <- function(data, base=NULL, extra=FALSE, verbose=verbose) {
     lev <- colnames(Y)
     MoP <- TRUE
     if(!is.null(base))
-      stop("Error: The last column of the response matrix must be the base category.\n No need to specify `base.'") 
+      stop("Error: The last column of the response matrix must be the base category.\n No need to specify `base.'")
     base <- lev[p]
-  } else { # standard Multinomial Probit model        
+  } else { # standard Multinomial Probit model
     Y <- as.factor(Y)
     lev <- levels(Y)
     if (!is.null(base))
       if (base %in% lev) {
-        Y <- relevel(Y, ref = base)
+        Y <- stats::relevel(Y, ref = base)
         lev <- levels(Y)
       } else {
         stop(paste("Error: `base' does not exist in the response variable."))
