@@ -1,40 +1,41 @@
 /******************************************************************
-  This file is a part of MNP: R Package for Estimating the 
+  This file is a part of MNP: R Package for Estimating the
   Multinomial Probit Models by Kosuke Imai and David A. van Dyk.
   Copyright: GPL version 2 or later.
 *******************************************************************/
 
 #include <string.h>
 #include <stddef.h>
-#include <stdio.h>      
+#include <stdio.h>
 #include <math.h>
 #include <Rmath.h>
 #include <R.h>
+#include <Rinternals.h>
 #include "vector.h"
 #include "subroutines.h"
 #include "rand.h"
 
-void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen, 
+void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
 	       double *b0,    /* prior mean for beta */
-	       double *pdA0, int *piNu0, double *pdS, double *pdX, 
+	       double *pdA0, int *piNu0, double *pdS, double *pdX,
 	       int *y,        /* response variable: -1 for missing */
-	       double *pdbeta, double *pdSigma, int *piImp, 
+	       double *pdbeta, double *pdSigma, int *piImp,
 	       int *invcdf,   /* use inverse cdf for TruncNorm? */
 	       int *piBurnin, /* the number of burnin */
 	       int *piKeep,
 	       int *itrace,
-	       int *verbose,  /* 1 if extra print is needed */ 
+	       int *verbose,  /* 1 if extra print is needed */
 	       int *piMoP,    /* 1 if Multinomial ordered Probit */
 	       int *latent,   /* 1 if W is stored */
 	       double *pdStore){
-  
+
   /* paramerters from R */
   int n_samp = *piNSamp; /* sample size */
   int n_gen = *piNGen;   /* number of gibbs draws */
   int n_cov = *piNCov;   /* The number of covariates */
   int n_dim = *piNDim;   /* The number of indentifiable dimension (p-1) */
   int nu0 = *piNu0;      /* nu0: degree of freedom, S: scale matrix */
-  int imp = *piImp;      /* 0: improper prior for beta - algorithms 1 & 2 
+  int imp = *piImp;      /* 0: improper prior for beta - algorithms 1 & 2
 			    1: proper prior for beta */
   int keep = 1;          /* counter for keep */
   int progress = 1;      /* counter for progress report */
@@ -66,7 +67,7 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
   int i, j, k, l, main_loop; /* used for loops */
 
   /* temporay storages */
-  int itemp, itempMax, itempMin, *ivtemp, itempS = 0, itempP=ftrunc((double) n_gen/10); 
+  int itemp, itempMax, itempMin, *ivtemp, itempS = 0, itempP=ftrunc((double) n_gen/10);
   double *vtemp;
   double **mtemp,**mtemp1,**mtemp2;
 
@@ -100,7 +101,7 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
   /** Packing Y, X, A0, S, beta, Sigma  **/
   if(*piMoP) {
     itemp = 0;
-    for (j = 0; j <= n_dim; j++) 
+    for (j = 0; j <= n_dim; j++)
       for (i = 0; i < n_samp; i++) Y[i][j] = y[itemp++];
     for (i = 0; i < n_samp; i++) { /* recording the max of Y[i] */
       for (j=0; j<= n_dim; j++)
@@ -111,29 +112,29 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
   }
 
   itemp = 0;
-  for (k = 0; k < n_cov; k++) 
-    for (j = 0; j < n_dim; j++) 
-      for (i = 0; i < n_samp; i++) 
+  for (k = 0; k < n_cov; k++)
+    for (j = 0; j < n_dim; j++)
+      for (i = 0; i < n_samp; i++)
 	X[i*n_dim+j][k] = pdX[itemp++];
-      
+
   /* PdoubleMatrix(X, n_dim*3, n_cov); */
   itemp = 0;
-  for (k = 0; k < n_cov; k++) 
+  for (k = 0; k < n_cov; k++)
     for (j = 0; j < n_cov; j++)	A0[j][k] = pdA0[itemp++];
 
   itemp = 0;
-  for (k = 0; k < n_dim; k++) 
+  for (k = 0; k < n_dim; k++)
     for (j = 0; j < n_dim; j++) S[j][k] = pdS[itemp++];
 
   itemp = 0;
-  for (j=0; j<n_cov;j++) 
+  for (j=0; j<n_cov;j++)
     beta[j]=pdbeta[itemp++];
 
   itemp = 0;
-  for (k = 0; k < n_dim; k++) 
-    for (j = 0; j < n_dim; j++) 
+  for (k = 0; k < n_dim; k++)
+    for (j = 0; j < n_dim; j++)
       Sigma[j][k] = pdSigma[itemp++];
-  dinv(Sigma,n_dim,SigInv); 
+  dinv(Sigma,n_dim,SigInv);
 
   /** add prior information as additional data points **/
   if(imp){
@@ -145,8 +146,8 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
     dcholdc(A0,n_cov,mtemp); /* Cholesky decomposition */
     for(j=0;j<n_cov;j++){
       Xstar[n_samp*n_dim+j][n_cov]=b0[j];
-      for(k=0;k<n_cov;k++){ 
-	Xstar[n_samp*n_dim+j][n_cov]+=mtemp[j][k]*b0[k]; 
+      for(k=0;k<n_cov;k++){
+	Xstar[n_samp*n_dim+j][n_cov]+=mtemp[j][k]*b0[k];
 	Xstar[n_samp*n_dim+j][k]=mtemp[j][k];
       }
     }
@@ -169,7 +170,7 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
   for(main_loop=1; main_loop<=n_gen; main_loop++){
     /* for Algorithm 1 (Scheme 1), draw alpha2 first */
     ss=0;
-    for(j=0;j<n_dim;j++)      
+    for(j=0;j<n_dim;j++)
       for(k=0;k<n_dim;k++) mtemp1[j][k]=0;
     for(i=0;i<n_dim;i++)
       for(j=0;j<n_dim;j++)
@@ -180,9 +181,9 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
     /** permutation of Sigma **/
     for(j=0;j<n_dim;j++){
       kindx = 0;
-      for(k=0;k<n_dim;k++){ 
+      for(k=0;k<n_dim;k++){
 	lindx=0;
-	for(l=0;l<n_dim;l++){ 
+	for(l=0;l<n_dim;l++){
 	  if(j!=k)
 	    if(j!=l)
 	      PerSig[j][k+kindx][l+lindx]=Sigma[k][l];
@@ -210,11 +211,11 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
 	for(j=0;j<n_dim;j++){
 	  Xbeta[i][j]=0;
 	  for (k=0;k<n_cov;k++) Xbeta[i][j]+=X[i*n_dim+j][k]*beta[k];
-	} 
+	}
 	for(j=0;j<n_dim;j++){
 	  if(Y[i][j]!=-1) {
 	    itempMax=0; itempMin=0;
-	    for (k=0;k<=n_dim;k++) 	  		  		  
+	    for (k=0;k<=n_dim;k++)
 	      if((j!=k) && (Y[i][k]!=-1)) {
 		if(Y[i][k]==(Y[i][j]+1)) {
 		  if(itempMax==0) {
@@ -234,23 +235,23 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
 		}
 	      }
 	  }
-	  itemp=0; 
-	  for (k=0;k<n_dim;k++) 
+	  itemp=0;
+	  for (k=0;k<n_dim;k++)
 	    if(j!=k) vtemp[itemp++]=W[i][k]-Xbeta[i][k];
 	  /* conditional mean and variance */
 	  cmean=Xbeta[i][j];
 	  cvar=1/PerSig[j][n_dim-1][n_dim-1];
-	  for(k=0;k<(n_dim-1);k++) 
+	  for(k=0;k<(n_dim-1);k++)
 	    cmean-=PerSig[j][n_dim-1][k]*vtemp[k]*cvar;
 	  /* sampling each W[i][j] conditionally on the other elements */
 	  /* printf("%5d%5d%5d%5d%5d%14g%14g%14g", i, Y[i][0], Y[i][1],
 	     Y[i][2], Y[i][3], W[i][0], W[i][1], W[i][2]); */
-	  if(Y[i][j]==-1) 
+	  if(Y[i][j]==-1)
 	    W[i][j] = cmean + norm_rand()*sqrt(cvar);
 	  else {
 	    if(Y[i][j]==0) minw = cmean - 1000*sqrt(cvar);
 	    if(Y[i][j]==maxy[i]) maxw = cmean + 1000*sqrt(cvar);
-	    W[i][j]=TruncNorm(minw,maxw,cmean,cvar,*invcdf); 
+	    W[i][j]=TruncNorm(minw,maxw,cmean,cvar,*invcdf);
 	  }
 	  /* printf("%14g\n", W[i][j]); */
 	  X[i*n_dim+j][n_cov]=W[i][j];
@@ -265,8 +266,8 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
 	  for(k=0;k<n_cov;k++) Xbeta[i][j]+=X[i*n_dim+j][k]*beta[k];
 	}
 	for(j=0;j<n_dim;j++){
-	  maxw=0.0; itemp=0; 
-	  for(k=0;k<n_dim;k++) {	  		  		  
+	  maxw=0.0; itemp=0;
+	  for(k=0;k<n_dim;k++) {
 	    if(j!=k) {
 	      maxw = fmax2(maxw, W[i][k]);
 	      /* if(maxw<W[i][k]) maxw=W[i][k]; */
@@ -276,13 +277,13 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
 	  /* conditional mean and variance */
 	  cmean=Xbeta[i][j];
 	  cvar=1/PerSig[j][n_dim-1][n_dim-1];
-	  for(k=0;k<(n_dim-1);k++) 
+	  for(k=0;k<(n_dim-1);k++)
 	    cmean-=PerSig[j][n_dim-1][k]*vtemp[k]*cvar;
 	  /* sampling each W[i][j] conditionally on the other elements */
 	  if(y[i]==-1)
 	    W[i][j]=cmean+norm_rand()*sqrt(cvar);
-	  else if(y[i]==(j+1)) 
-	    W[i][j]=TruncNorm(maxw,cmean+1000*sqrt(cvar),cmean,cvar,*invcdf); 
+	  else if(y[i]==(j+1))
+	    W[i][j]=TruncNorm(maxw,cmean+1000*sqrt(cvar),cmean,cvar,*invcdf);
 	  else
 	    W[i][j]=TruncNorm(cmean-1000*sqrt(cvar),maxw,cmean,cvar,*invcdf);
 	  X[i*n_dim+j][n_cov]=W[i][j];
@@ -290,58 +291,58 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
 	}
       }
     }
-    
+
     /* trace(S*SigInv) */
     ss=0;
     for(j=0;j<n_dim;j++)
-      for(k=0;k<n_dim;k++) mtemp1[j][k]=0; 
+      for(k=0;k<n_dim;k++) mtemp1[j][k]=0;
     for(i=0;i<n_dim;i++)
       for(j=0;j<n_dim;j++)
 	for(k=0;k<n_dim;k++) mtemp1[j][k]+=S[j][i]*SigInv[i][k];
     for(j=0;j<n_dim;j++) ss+=mtemp1[j][j];
-    
+
     /* multiply X and W by the Inverse of the Cholesky factor */
     dcholdc(SigInv,n_dim,mtemp1);
     for(i=0;i<n_samp*n_dim;i++)
       for(j=0;j<=n_cov;j++) Xstar[i][j]=0;
     for(i=0;i<n_samp;i++)
       for(j=0;j<n_dim;j++)
-	for(k=0;k<n_dim;k++) 
+	for(k=0;k<n_dim;k++)
 	  for(l=0;l<=n_cov;l++)
 	    Xstar[i*n_dim+k][l]+=mtemp1[j][k]*X[i*n_dim+j][l];
-    
+
     /* construct SS matrix for SWEEP */
     for(j=0;j<=n_cov;j++)
       for(k=0;k<=n_cov;k++) SS[j][k]=0;
     for(i=0;i<n_samp;i++)
       for(j=0;j<n_dim;j++)
 	for(k=0;k<=n_cov;k++)
-	  for(l=0;l<=n_cov;l++) 
+	  for(l=0;l<=n_cov;l++)
 	    SS[k][l]+=Xstar[i*n_dim+j][k]*Xstar[i*n_dim+j][l];
     for(j=0;j<n_cov;j++)
       for(k=0;k<=n_cov;k++)
-	for(l=0;l<=n_cov;l++) 
+	for(l=0;l<=n_cov;l++)
 	  SS[k][l]+=Xstar[n_samp*n_dim+j][k]*Xstar[n_samp*n_dim+j][l];
-    
+
     /* SWEEP to get posterior mean and variance for beta */
     for(j=0;j<n_cov;j++) SWP(SS,j,n_cov+1);
     /* draw alpha2 given Sigma and W */
-    ss+=SS[n_cov][n_cov];   
+    ss+=SS[n_cov][n_cov];
     alpha2=ss/(double)rchisq((double)(n_samp+nu0)*n_dim);
-    
+
     /* draw beta given Sigma and W */
-    for(j=0;j<n_cov;j++){ 
+    for(j=0;j<n_cov;j++){
       mbeta[j]=SS[j][n_cov];
       for(k=0;k<n_cov;k++) Vbeta[j][k]=-SS[j][k]*alpha2;
     }
-    rMVN(beta,mbeta,Vbeta,n_cov);  
+    rMVN(beta,mbeta,Vbeta,n_cov);
     /* draw Sigma given beta and W */
     for(i=0;i<n_samp;i++)
       for(j=0;j<n_dim;j++){
-	epsilon[i*n_dim+j]=X[i*n_dim+j][n_cov]; 
+	epsilon[i*n_dim+j]=X[i*n_dim+j][n_cov];
 	for(k=0;k<n_cov;k++)
 	  epsilon[i*n_dim+j]-=X[i*n_dim+j][k]*beta[k];
-      } 
+      }
     for(j=0;j<n_dim;j++)
       for(k=0;k<n_dim;k++) R[j][k]=0;
     for(i=0;i<n_samp;i++)
@@ -352,12 +353,12 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
     dinv(mtemp1,n_dim,mtemp2);
     rWish(SigInv,mtemp2,nu0+n_samp,n_dim);
     dinv(SigInv,n_dim,Sigma);
-    
+
     /* recompute some quantities using the updated alpha2 */
     for(j=0;j<n_cov;j++) beta[j]/=sqrt(alpha2);
     if (*itrace) {
       alpha2=0;
-      for(k=0;k<n_dim; k++) 
+      for(k=0;k<n_dim; k++)
 	alpha2+=Sigma[k][k];
       alpha2 = alpha2/n_dim;
     } else {
@@ -371,20 +372,20 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
     for(i=0;i<n_samp;i++)
       for(j=0;j<n_dim;j++)
 	W[i][j]=X[i*n_dim+j][n_cov]/sqrt(alpha2);
-  
+
     /* print Gibbs draws for all the schmes! */
     R_CheckUserInterrupt();
     if(main_loop > *piBurnin) {
       if(keep==*piKeep) {
-	for(j=0;j<n_cov;j++) 
+	for(j=0;j<n_cov;j++)
 	  pdStore[itempS++]=beta[j];
-	for(j=0;j<n_dim;j++) 
-	  for(k=0;k<n_dim;k++) 
-	    if(j<=k) 
+	for(j=0;j<n_dim;j++)
+	  for(k=0;k<n_dim;k++)
+	    if(j<=k)
 	      pdStore[itempS++]=Sigma[j][k];
-	if (*latent) 
-	  for (i=0;i<n_samp;i++) 
-	    for (j=0;j<n_dim;j++) 
+	if (*latent)
+	  for (i=0;i<n_samp;i++)
+	    for (j=0;j<n_dim;j++)
 	      pdStore[itempS++]=W[i][j];
 	keep=1;
       }
@@ -395,14 +396,14 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
       if(main_loop == itempP) {
 	Rprintf("%3d percent done.\n", progress*10);
 	itempP+=ftrunc((double) n_gen/10); progress++;
-	R_FlushConsole(); 
+	R_FlushConsole();
       }
     }
   } /* end of Gibbs sampler */
-  
+
   /** write out the random seed **/
   PutRNGstate();
-  
+
   /** freeing memory **/
   if(*piMoP)
     FreeintMatrix(Y, n_samp);
@@ -432,12 +433,12 @@ void cMNPgibbs(int *piNDim, int *piNCov, int *piNSamp, int *piNGen,
 
 
 /* unitility function */
-void R_max_col2(double **matrix, 
-	       int nr, 
-	       int nc, 
-	       int *maxes, 
+void R_max_col2(double **matrix,
+	       int nr,
+	       int nc,
+	       int *maxes,
 	       int ties_meth) {
-  
+
   int *ncol = intArray(1);
   int *nrow = intArray(1);
   int *ties = intArray(1);
@@ -459,9 +460,9 @@ void R_max_col2(double **matrix,
   free(tmp);
 
 }
- 
 
-void predict(double *dX,     /* X matrix */
+
+void predictP(double *dX,     /* X matrix */
 	     int *nobs,      /* number of observations */
 	     double *dcoef,  /* coefficients */
 	     double *dSigma, /* covariances */
@@ -479,7 +480,7 @@ void predict(double *dX,     /* X matrix */
   int n_draw = *ndraws;
   int n_dim = *ndims;
   int n_cov = *ncovs;
-  int n_extra = *moredraws; 
+  int n_extra = *moredraws;
 
   double **X = doubleMatrix(n_samp*n_dim, n_cov);
   double *Xbeta = doubleArray(n_cov);
@@ -491,7 +492,7 @@ void predict(double *dX,     /* X matrix */
 
   int i, j, k, main_loop, itemp, itempP, itempO, itempC;
   int total = n_extra*n_samp*n_draw;
-  int count, progress = 1; 
+  int count, progress = 1;
   int itempQ = ftrunc((double) total/10);
   int *maxdim = intArray(n_extra);
   int *ind = intArray(n_dim+1);
@@ -500,17 +501,17 @@ void predict(double *dX,     /* X matrix */
 
   /** reading the data */
   itemp = 0;
-  for (k = 0; k < n_cov; k++) 
-    for (j = 0; j < n_dim; j++) 
-      for (i = 0; i < n_samp; i++) 
-	X[i*n_dim+j][k] = dX[itemp++];  
+  for (k = 0; k < n_cov; k++)
+    for (j = 0; j < n_dim; j++)
+      for (i = 0; i < n_samp; i++)
+	X[i*n_dim+j][k] = dX[itemp++];
 
   /** reading the MCMC draws **/
   itemp = 0;
   for (k = 0; k < n_cov; k++)
     for (j = 0; j < n_draw; j++)
       beta[j][k] = dcoef[itemp++];
-  
+
   itemp = 0;
   for (k = 0; k < n_dim; k++)
     for (j = k; j < n_dim; j++)
@@ -523,19 +524,19 @@ void predict(double *dX,     /* X matrix */
   GetRNGstate();
 
   /** Posterior predictive simulations **/
-  itempC = 0; itempO = 0; itempP = 0; count = 0; 
+  itempC = 0; itempO = 0; itempP = 0; count = 0;
   itempQ = 0;
 
   /* loop for observations */
-  for (i = 0; i < n_samp; i++) { 
+  for (i = 0; i < n_samp; i++) {
     if (n_extra == 1) {
-      for (j = 0; j <= n_dim; j++) 
+      for (j = 0; j <= n_dim; j++)
 	probTemp[j] = 0;
     }
     /* loop for MCMC draws */
     for (main_loop = 0; main_loop < n_draw; main_loop++) {
       if (n_extra > 1) {
-	for (j = 0; j <= n_dim; j++) 
+	for (j = 0; j <= n_dim; j++)
 	  probTemp[j] = 0;
       }
       /* compute the mean for each dimension */
@@ -553,7 +554,7 @@ void predict(double *dX,     /* X matrix */
 	  W[j][k+1] = vtemp[k];
 	W[j][0] = 0;
       }
-      /* which dimension is max for each of n_extra W draws? 
+      /* which dimension is max for each of n_extra W draws?
 	 PdoubleMatrix(W, n_extra, n_dim+1); */
       R_max_col2(W, n_extra, n_dim+1, maxdim, 1);
       /* PintArray(maxdim, n_extra); */
@@ -570,7 +571,7 @@ void predict(double *dX,     /* X matrix */
 	  if(count == itempQ) {
 	    Rprintf("%3d percent done.\n", progress*10);
 	    itempQ += ftrunc((double) total/10); progress++;
-	    R_FlushConsole(); 
+	    R_FlushConsole();
 	  }
 	  count++;
 	}
