@@ -1,17 +1,17 @@
 #include <stdlib.h> // for NULL
 #include <R_ext/Rdynload.h>
 
-/* FIXME: 
+/* FIXME:
    Check these declarations against the C/Fortran source code.
 */
 
 /* .C calls */
 extern void cMNPgibbs(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
-extern void predict(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
+extern void predictP(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 
 static const R_CMethodDef CEntries[] = {
     {"cMNPgibbs", (DL_FUNC) &cMNPgibbs, 21},
-    {"predict",   (DL_FUNC) &predict,   12},
+    {"predictP",   (DL_FUNC) &predictP,   12},
     {NULL, NULL, 0}
 };
 
